#include <iostream>
#include <sstream>
#include "Game.h"
#include "Define.h"
#include "Level\Level.h"
#include <graphics/Texture.h>
#include "Screen/MainScreen.h"
#include "Screen/Instructions.h"

using namespace std;

Game::Game(void)
{
}


Game::~Game(void)
{
}

Game* Game::Get()
{
	static Game game;
	return &game;
}

bool Game::InitGame()
{
	if( !m_DefaultFont.LoadFromFile("resources/OCRAExt.ttf"))
	{
		cerr << "[ERROR] " << __FILE__ << ":" << __LINE__ << " Loading of font resources/OCRAExt.ttf failed." << endl;
		m_DefaultFont = Font::GetDefaultFont();
	}

	if( !m_imgInterface.LoadFromFile("resources/interface.png"))
	{
		cerr << "[ERROR] " << __FILE__ << ":" << __LINE__ << " Loading of font resources/interface.png failed." << endl;
	}

	ostringstream oss;
	Image *pImg = NULL;
	CTexture *pText = NULL;
	// Load path images
	for( int i = 0; i < 11; i++)
	{
		oss << "resources/path";
		if( i < 10 )
			oss << "0";
		oss << i << ".png";

		pImg = new Image();
		pImg->LoadFromFile(oss.str());
		m_TabImgPath.push_back(pImg);

		pText = new CTexture(0, 0, pImg->GetWidth(), pImg->GetHeight(), *pImg);
		pText->SetScale(2.f, 2.f);
		m_TabTextPath.push_back(pText);

		oss.str("");
	}

	// Load obstacle images
	for( int i = 0; i < 9; i++)
	{
		oss << "resources/obstacle";
		if( i < 10 )
			oss << "0";
		oss << i << ".png";

		pImg = new Image();
		pImg->LoadFromFile(oss.str());
		m_TabImgObs.push_back(pImg);

		pText = new CTexture(0, 0, pImg->GetWidth(), pImg->GetHeight(), *pImg);
		m_TabTextObs.push_back(pText);

		oss.str("");
	}

	m_State = MAIN_SCREEN;

	m_MainScreen = new MainScreen();
	m_Instructions = new Instructions();
	m_Level = new Level();

	return true;
}

void Game::DeInitGame()
{
}


bool Game::FrameMove()
{
	CEngine::FrameMove();

	const Input& Input = m_Screen.GetInput();
	unsigned int x = Input.GetMouseX();
	unsigned int y = Input.GetMouseY();

	switch(m_State)
	{
	case MAIN_SCREEN:
		m_MainScreen->FrameMove(m_elapsedTime);
		break;
	case GAME:
		MouseOver(x, y);
		m_Level->FrameMove(m_elapsedTime);
		break;
	case INSTRUCTIONS:
		m_Instructions->FrameMove(m_elapsedTime);
		break;
	}

	return true;
}

bool Game::Render()
{
	m_Screen.Clear();

	switch(m_State)
	{
	case MAIN_SCREEN:
		m_MainScreen->Render(m_Screen);
		break;
	case GAME:
		m_Level->Render(m_Screen);
		break;
	case INSTRUCTIONS:
		m_Instructions->Render(m_Screen);
		break;
	}

	m_Screen.Display();

	return true;
}

// Events =====================================
bool Game::ExecuteAction( int action )
{
	if( action == CMD_NONE)
		return true;

	bool noExit = true;

	switch(m_State)
	{
	case MAIN_SCREEN:
		switch(action)
		{
		case CMD_PLAY:	// Launch the game
			m_Level->Init();
			m_State = GAME;
			break;
		case CMD_INSTRUCTIONS:	// Display game instructions
			m_State = INSTRUCTIONS;
			break;
		case CMD_EXIT:	// Exit the application
			noExit = false;
			break;
		}
		break;
	case GAME:
		switch(action)
		{
		case CMD_VICTORY:
			cout << "Victory !!" << endl;
			break;
		case CMD_DEFEAT:
			cout << "Defeat !!" << endl;
			break;
		case CMD_EXIT:
			m_State = MAIN_SCREEN;
			m_Level->DeInit();
			break;
		}
		break;
	case INSTRUCTIONS:
		switch(action)
		{
		case CMD_EXIT:
			m_State = MAIN_SCREEN;
			break;
		}
		break;
	}

	return noExit;
}

bool Game::KeyPressed( Key::Code key )
{
	switch(m_State)
	{
	case MAIN_SCREEN:
		break;
	case GAME:
		if( key == Key::Escape )
		{
			ExecuteAction(CMD_EXIT);
		}
		break;
	case INSTRUCTIONS:
		if( key == Key::Escape )
		{
			ExecuteAction(CMD_EXIT);
		}
		break;
	}

	return true;
}

bool Game::KeyUnPressed( Key::Code/* key*/ )
{

	return true;
}

bool Game::MousePressed( Event mouseButton )
{
	const Input& Input = m_Screen.GetInput();
	unsigned int x = Input.GetMouseX();
	unsigned int y = Input.GetMouseY();
	Vector2f mousePos = m_Screen.ConvertCoords(x, y);

	switch(m_State)
	{
	case MAIN_SCREEN:
		m_MainScreen->MousePressed(static_cast<int>(mousePos.x),
			static_cast<int>(mousePos.y));
		break;
	case GAME:
		switch(mouseButton.MouseButton.Button)
		{
		case Mouse::Right:
			m_Level->Forward();
			break;
		case Mouse::Left:
			m_Level->Reward();
			ExecuteAction(m_Level->MousePressed(mousePos));
			break;
		}
		break;
	case INSTRUCTIONS:
		ExecuteAction(CMD_EXIT);
		break;
	}

	return true;
}
bool Game::MouseReleased( Event /*mouseButton*/ )
{
	bool result = true;

	const Input& Input = m_Screen.GetInput();
	unsigned int x = Input.GetMouseX();
	unsigned int y = Input.GetMouseY();
	Vector2f mousePos = m_Screen.ConvertCoords(x, y);

	switch(m_State)
	{
	case MAIN_SCREEN:
		result = ExecuteAction(m_MainScreen->MouseReleased(static_cast<int>(mousePos.x),
			static_cast<int>(mousePos.y)));
		break;
	case GAME:
		break;
	case INSTRUCTIONS:
		break;
	}

	return result;
}

bool Game::MouseOver( int x, int y )
{
	const Input& Input = m_Screen.GetInput();
	x = Input.GetMouseX();
	y = Input.GetMouseY();
	Vector2f mousePos = m_Screen.ConvertCoords(x, y);

	switch(m_State)
	{
	case MAIN_SCREEN:
		m_MainScreen->MouseHover(static_cast<int>(mousePos.x),
			static_cast<int>(mousePos.y));
		break;
	case GAME:
		mousePos = m_Screen.ConvertCoords(x, y, m_Level->GetView());
		ExecuteAction(m_Level->MouseMove(mousePos));
		break;
	case INSTRUCTIONS:
		break;
	}

	return true;
}

bool Game::TextEntered( Event::TextEvent /*text*/)
{
	return true;
}


bool Game::MouseWheelMoved( Event::MouseWheelEvent wheel)
{
	if( m_State == GAME )
	{
		if( wheel.Delta > 0 )
		{
			for( int i = 0; i < wheel.Delta; i++)
				m_Level->Forward();
		}
		else
		{	
			for( int i = 0; i > wheel.Delta; i--)
				m_Level->Reward();
		}
	}

	return true;
}

const Font& Game::GetDefaultFont()
{
	return m_DefaultFont;
}

CTexture* Game::GetPathTexture(int index)
{
	if( index < 0 || index >= static_cast<int>(m_TabTextPath.size()))
		return NULL;

	return m_TabTextPath[index];
}

CTexture* Game::GetObsTexture(int index)
{
	if( index < 0 || index >= static_cast<int>(m_TabTextObs.size()))
		return NULL;

	return m_TabTextObs[index];
}
