#include "Level.h"
#include "../Define.h"
#include <iostream>
#include <sstream>
#include "../Game.h"
#include "../Obstacle/Obstacle.h"
#include "../Obstacle/SpinningObstacle.h"
#include "../Obstacle/MovingObstacle.h"
#include <iomanip>
#include <graphics/ProgressBar.h>
#include <graphics/FrameImage.h>
#include <graphics/Frame.h>

using namespace std;

const float Level::SPEED_INTERVAL = 0.5f;

Level::Level(void):
m_View			(Vector2f(SCREEN_WIDTH / 2.f, SCREEN_HEIGHT / 2.f),
	Vector2f(static_cast<float>(SCREEN_WIDTH), static_cast<float>(SCREEN_HEIGHT))),
m_ScrollSpeed	(100.f),
m_SpeedFactor	(1.f),
m_RunChrono		(true),
m_CurrentIndex	(0),
m_PrevPath		(NULL),
m_CurrentPath	(NULL),
m_NextPath		(NULL),
m_CurrentRFEffect	(NULL),
m_imgProgressBar	(NULL),
m_ProgressBar		(NULL),
m_GoldTimeRef		(60.f),
m_SilverTimeRef		(70.f),
m_BronzeTimeRef		(85.f)
{
	m_txtTime.SetFont(Game::Get()->GetDefaultFont());
	m_txtTime.SetCharacterSize(30);
	m_txtTime.SetPosition(10.f, 10.f);
	m_txtTime.SetStyle(Text::Italic);

	m_txtFactor.SetFont(Game::Get()->GetDefaultFont());
	m_txtFactor.SetCharacterSize(30);
	m_txtFactor.SetPosition(SCREEN_WIDTH - 240.f, 10.f);
	m_txtFactor.SetStyle(Text::Italic);

	m_ForwardImg.LoadFromFile("resources/forward.png");
	m_Forward.SetImage(m_ForwardImg);
	m_Forward.SetOrigin(m_ForwardImg.GetWidth() / 2.f,
		m_ForwardImg.GetHeight() / 2.f);
	m_Forward.SetPosition(SCREEN_WIDTH / 2.f,
		SCREEN_HEIGHT / 2.f);

	m_RewardImg.LoadFromFile("resources/reward.png");
	m_Reward.SetImage(m_RewardImg);
	m_Reward.SetOrigin(m_RewardImg.GetWidth() / 2.f,
		m_RewardImg.GetHeight() / 2.f);
	m_Reward.SetPosition(SCREEN_WIDTH / 2.f,
		SCREEN_HEIGHT / 2.f);

	// Progress bar
	m_imgProgressBar = new FrameImage(0, 0, 7, 7, 3, 3, 3, 3, Game::Get()->m_imgInterface);
	m_Bar.SetImage(Game::Get()->m_imgInterface);
	m_Bar.SetSubRect(IntRect(7, 0, 1, 7));
	m_ProgressBar = new ProgressBar(50.f, SCREEN_HEIGHT - 40.f,
		SCREEN_WIDTH - 100.f, 12.f,
		m_imgProgressBar,
		m_Bar,
		0.f);

	// Go Button
	m_btnGo = CTexture(0, 39, 100, 100, Game::Get()->m_imgInterface);
	m_btnGo.SetPosition(SCREEN_WIDTH / 2.f,
		SCREEN_HEIGHT / 2.f);
	m_btnGo.SetOrigin(100 / 2.f,
		100 / 2.f);

	m_txtGold.SetFont(Game::Get()->GetDefaultFont());
	m_txtGold.SetColor(Color::Yellow);
	m_txtGold.SetPosition(100.f, 70.f);
	m_txtGold.SetString("Gold Medal: 01\"00'00");

	m_txtSilver.SetFont(Game::Get()->GetDefaultFont());
	m_txtSilver.SetColor(Color(220, 220, 220));
	m_txtSilver.SetPosition(100.f, 120.f);
	m_txtSilver.SetString("Silver Medal: 01\"10'00");

	m_txtBronze.SetFont(Game::Get()->GetDefaultFont());
	m_txtBronze.SetColor(Color(0x8C, 0x78, 0x53));
	m_txtBronze.SetPosition(100.f, 170.f);
	m_txtBronze.SetString("Bronze Medal: 01\"25'00");

	// Frames
	m_TimeFrame = new Frame(96.f, 69.f, 420, 45, m_imgProgressBar);
	m_ChronoFrame = new Frame(5.f, 12.f, 160, 38, m_imgProgressBar);
	m_SpeedFrame = new Frame(SCREEN_WIDTH - 250.f, 12.f, 245, 38, m_imgProgressBar);

	m_imgBlack.Create(1, 1, Color::Black);
	m_Fading.SetImage(m_imgBlack);
	m_Fading.SetScale(static_cast<float>(SCREEN_WIDTH), static_cast<float>(SCREEN_HEIGHT));
	m_Fading.SetColor(Color(255, 255, 255, 0));
}

Level::~Level(void)
{
	delete m_SpeedFrame;
	delete m_ChronoFrame;
	delete m_TimeFrame;
	delete m_imgProgressBar;
	delete m_ProgressBar;
}

bool Level::Init()
{
	m_View.SetCenter(Vector2f(SCREEN_WIDTH / 2.f, SCREEN_HEIGHT / 2.f));
	m_SpeedFactor = 1.0f;
	m_ElapsedTime = 0.f;
	m_txtTime.SetString("00\"00'00");
	m_txtFactor.SetString("Speed x 1.0");
	m_CurrentIndex = 0;
	m_ElapsedDistance = 0.f;
	m_RunChrono = true;
	m_Defeat = false;
	m_DefeatTimer = 0.f;

	// Instance de niveau =======================
	m_TabPathId.push_back(0);	// 0
	m_lObstacles.push_back(new Obstacle(new CTexture(*Game::Get()->GetObsTexture(0)), Vector2f(400.f + 1 * 800.f, 300.f)));
	m_TabPathId.push_back(0);	// 1
	m_TabPathId.push_back(6);	// 2
	m_TabPathId.push_back(2);	// 3
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(1)), Vector2f(400.f + 4 * 800.f, 300.f), 30.f));
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(2)), Vector2f(400.f + 4 * 800.f, 300.f), -30.f));
	m_TabPathId.push_back(0);	// 4
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(400.f + 5 * 800.f, 50.f),
		Vector2f(0.f, 80.f),
		0.f, 200.f,
		false));
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(400.f + 5 * 800.f, 350.f),
		Vector2f(0.f, 80.f),
		0.f, 200.f,
		false));
	m_TabPathId.push_back(9);	// 5
	m_lObstacles.push_back(new Obstacle(new CTexture(*Game::Get()->GetObsTexture(3)), Vector2f(400.f + 6 * 800.f, 300.f)));
	m_TabPathId.push_back(7);	// 6
	m_TabPathId.push_back(0);	// 7
	m_TabPathId.push_back(3);	// 8
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(6)), Vector2f(400.f + 9 * 800.f, 300.f), -50.f));
	m_TabPathId.push_back(7);	// 9
	m_TabPathId.push_back(0);	// 10
	m_TabPathId.push_back(8);	// 11
	m_TabPathId.push_back(0);	// 12
	m_TabPathId.push_back(6);	// 13
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(400.f + 14 * 800.f, 300.f),
		Vector2f(0.f, 80.f),
		150.f, 150.f,
		false));
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(15 * 800.f, 300.f),
		Vector2f(0.f, -80.f),
		150.f, 150.f,
		false));
	m_TabPathId.push_back(0);	// 14
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(400.f + 15 * 800.f, 300.f),
		Vector2f(0.f, 80.f),
		150.f, 150.f,
		false));
	m_TabPathId.push_back(0);	// 15
	m_TabPathId.push_back(2);	// 16
	m_TabPathId.push_back(0);	// 17
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(5)),
		Vector2f(200.f + 18 * 800.f, 300.f),
		Vector2f(100.f, 0.f),
		0.f, 400.f,
		true));
	m_TabPathId.push_back(3);	// 18
	m_TabPathId.push_back(4);	// 19
	m_TabPathId.push_back(0);	// 20
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(7)), Vector2f(300.f + 21 * 800.f, 300.f), -30.f));
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(7)), Vector2f(400.f + 21 * 800.f, 300.f), 30.f));
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(7)), Vector2f(500.f + 21 * 800.f, 300.f), -30.f));
	m_TabPathId.push_back(10);	// 21

	m_TabPathId.push_back(6);	// 22
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(6)), Vector2f(400.f + 23 * 800.f, 300.f), -50.f));
	m_TabPathId.push_back(7);	// 23
	m_TabPathId.push_back(0);	// 24
	m_TabPathId.push_back(3);	// 25
	m_TabPathId.push_back(2);	// 26
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(1)), Vector2f(400.f + 27 * 800.f, 300.f), 30.f));
	m_TabPathId.push_back(0);	// 27
	m_TabPathId.push_back(0);	// 28
	m_lObstacles.push_back(new SpinningObstacle(new CTexture(*Game::Get()->GetObsTexture(2)), Vector2f(400.f + 29 * 800.f, 300.f), -30.f));
	m_TabPathId.push_back(0);	// 29
	m_TabPathId.push_back(5);	// 30
	m_TabPathId.push_back(6);	// 31
	m_TabPathId.push_back(0);	// 32
	m_TabPathId.push_back(9);	// 33
	m_TabPathId.push_back(0);	// 34
	m_TabPathId.push_back(7);	// 35
	m_TabPathId.push_back(0);	// 36
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(200.f + 37 * 800.f, 300.f),
		Vector2f(0.f, -80.f),
		160.f, 160.f,
		false));
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(300.f + 37 * 800.f, 300.f),
		Vector2f(0.f, 80.f),
		160.f, 160.f,
		false));
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(400.f + 37 * 800.f, 300.f),
		Vector2f(0.f, -80.f),
		160.f, 160.f,
		false));
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(500.f + 37 * 800.f, 300.f),
		Vector2f(0.f, 80.f),
		160.f, 160.f,
		false));
	m_lObstacles.push_back(new MovingObstacle(new CTexture(*Game::Get()->GetObsTexture(4)),
		Vector2f(600.f + 37 * 800.f, 300.f),
		Vector2f(0.f, -80.f),
		160.f, 160.f,
		false));
	m_TabPathId.push_back(10);	// 37
	m_TabPathId.push_back(6);	// 38

	// Fin
	m_TabPathId.push_back(1);

	// =======================

	m_CurrentPath = new CTexture(*Game::Get()->GetPathTexture(m_TabPathId[0]));
	m_CurrentPath->SetPosition(0.f, 0.f);

	m_NextPath = new CTexture(*Game::Get()->GetPathTexture(m_TabPathId[1]));
	m_NextPath->SetPosition(800.f, 0.f);

	m_RFEffect = false;
	m_TimerEffect = 0.f;

	m_ProgressBar->SetMax((m_TabPathId.size() - 2) * 800.f);
	m_ProgressBar->SetElapsed(0.1f);

	m_Started = false;
	m_FadingGo = 0.f;
	m_btnGo.SetColor(Color::White);
	m_btnGo.SetScale(1.f, 1.f);

	return true;
}

void Level::DeInit()
{
	m_TabPathId.clear();
	for( list<Obstacle*>::iterator it = m_lObstacles.begin(); it != m_lObstacles.end(); it++)
	{
		delete *it;
	}
	m_lObstacles.clear();
}

bool Level::FrameMove(float elapsedTime)
{
	//////////////////////////////////////////////////
	// Non affected by speed factor
	ButtonGoEffect(elapsedTime);
	RFEffect(elapsedTime);

	if( m_Defeat )
	{
		// Fade in
		m_DefeatTimer -= elapsedTime;
		if( m_DefeatTimer < 0.f )
		{
			m_DefeatTimer = 0.01f;
			m_Defeat = false;
			DeInit();
			Init();
		}
		else
		{
			Color col = m_Fading.GetColor();
			col.a = static_cast<sf::Uint8>((0.4f - m_DefeatTimer) / 0.4f * 255);
			m_Fading.SetColor(col);
		}
	}
	else if( m_DefeatTimer < 0.4f)
	{
		// Fade out
		m_DefeatTimer += elapsedTime;
		if( m_DefeatTimer > 0.4f )
			m_DefeatTimer = 0.4f;

		Color col = m_Fading.GetColor();
		col.a = static_cast<sf::Uint8>((0.4f - m_DefeatTimer) / 0.4f * 255);
		m_Fading.SetColor(col);
	}

	if( m_Started && !m_Defeat )
	{
		if( m_RunChrono )
			RefreshChronometer(elapsedTime);

		//////////////////////////////////////////////////
		// Affected by speed factor
		elapsedTime *= m_SpeedFactor;

		m_View.Move(m_ScrollSpeed * elapsedTime, 0.f);
		m_ElapsedDistance += m_ScrollSpeed * elapsedTime;

		// Refresh the progress bar
		m_ProgressBar->SetElapsed(m_ElapsedDistance);

		// Next path ?
		if( m_ElapsedDistance > 800.f * (m_CurrentIndex + 1))
		{
			if( NextPath() )
				m_SpeedFactor = 0.f;
		}

		Obstacle *pObstacle = NULL;
		for( list<Obstacle*>::iterator it = m_lObstacles.begin(); it != m_lObstacles.end(); it++)
		{
			pObstacle = *it;
			pObstacle->FrameMove(elapsedTime);
		}
	}

	return true;
}

void Level::Render(RenderWindow &screen)
{
	screen.SetView(m_View);

	if( m_PrevPath )
		screen.Draw(*m_PrevPath);
	if( m_CurrentPath )
		screen.Draw(*m_CurrentPath);
	if( m_NextPath )
		screen.Draw(*m_NextPath);

	Obstacle *pObstacle = NULL;
	for( list<Obstacle*>::iterator it = m_lObstacles.begin(); it != m_lObstacles.end(); it++)
	{
		pObstacle = *it;
		pObstacle->Render(screen);
	}

	// Restore the default view
	screen.SetView(screen.GetDefaultView());
	m_ChronoFrame->Render(screen);
	screen.Draw(m_txtTime);
	m_SpeedFrame->Render(screen);
	screen.Draw(m_txtFactor);

	if( m_TimerEffect > 0.f)
		screen.Draw(*m_CurrentRFEffect);

	m_ProgressBar->Render(screen);

	m_btnGo.Render(screen);

	if( !m_Started )
	{
		m_TimeFrame->Render(screen, 0, 0);
		screen.Draw(m_txtGold);
		m_TimeFrame->Render(screen, 0, 50);
		screen.Draw(m_txtSilver);
		m_TimeFrame->Render(screen, 0, 100);
		screen.Draw(m_txtBronze);
	}

	screen.Draw(m_Fading);
}

int Level::MouseMove(Vector2f pos)
{
	int cmd = CMD_NONE;

	if( !m_Started || m_Defeat)
		return cmd;

	Obstacle *pObstacle = NULL;
	for( list<Obstacle*>::iterator it = m_lObstacles.begin(); it != m_lObstacles.end(); it++)
	{
		pObstacle = *it;
		if( pObstacle->Collide(pos.x, pos.y) )
		{
			cmd = CMD_DEFEAT;
			break;
		}
	}

	if( cmd == CMD_NONE )
	{
		Vector2f local;
		// Previous
		if(m_PrevPath)
		{
			local = m_PrevPath->TransformToLocal(pos);
			if( m_PrevPath->PointCollision(local) )
				cmd = CMD_DEFEAT;
		}
		// Current
		if( cmd == CMD_NONE )
		{
			local = m_CurrentPath->TransformToLocal(pos);
			if( m_CurrentPath->PointCollision(local) )
				cmd = CMD_DEFEAT;
		}
		// Next
		if( cmd == CMD_NONE && m_NextPath)
		{
			local = m_NextPath->TransformToLocal(pos);
			if( m_NextPath->PointCollision(local) )
			{
				if( m_CurrentIndex == static_cast<int>(m_TabPathId.size() - 2))
				{
					cmd = CMD_VICTORY;
					m_RunChrono = false;
				}
				else
					cmd = CMD_DEFEAT;
			}
		}
	}

	if( cmd == CMD_VICTORY )
		m_SpeedFactor = 0.f;
	else if( cmd == CMD_DEFEAT )
	{
		m_Defeat = true;
		m_DefeatTimer = 0.4f;
		m_RunChrono = false;
	}

	return cmd;
}

int Level::MousePressed(Vector2f pos)
{
	pos = m_btnGo.TransformToLocal(pos);
	if( !m_Started && m_btnGo.PointCollision(pos))
	{
		m_FadingGo = 1.f;
		m_Started = true;
	}

	return CMD_NONE;
}

// Up the speed
void Level::Forward()
{
	if( !m_Started || m_SpeedFactor >= 15.0f)
		return;

	m_SpeedFactor += SPEED_INTERVAL;
	if( m_SpeedFactor > 2.f )
		m_SpeedFactor += SPEED_INTERVAL;
	if( m_SpeedFactor > 5.f )
		m_SpeedFactor += SPEED_INTERVAL * 2;

	m_CurrentRFEffect = &m_Forward;
	m_TimerEffect = .5f;

	RefreshSpeedFactor();
}

// Down the speed
void Level::Reward()
{
	if( !m_Started || m_SpeedFactor == 0.5f)
		return;

	m_SpeedFactor -= SPEED_INTERVAL;
	if( m_SpeedFactor > 2.f )
		m_SpeedFactor -= SPEED_INTERVAL;
	if( m_SpeedFactor > 5.f )
		m_SpeedFactor -= SPEED_INTERVAL * 2;

	m_CurrentRFEffect = &m_Reward;
	m_TimerEffect = .5f;

	RefreshSpeedFactor();
}

const View& Level::GetView()
{
	return m_View;
}

void Level::RefreshChronometer(float elapsedTime)
{
	m_ElapsedTime += elapsedTime;

	int min = static_cast<int>(m_ElapsedTime / 60);
	int sec = static_cast<int>(m_ElapsedTime) % 60;
	int cs = static_cast<int>(m_ElapsedTime * 100) % 100;

	ostringstream oss;
	if( min < 10 )
		oss << "0";
	oss << min << "\"";
	if( sec < 10 )
		oss << "0";
	oss << sec << "'";
	if( cs < 10 )
		oss << "0";
	oss << cs;

	m_txtTime.SetString(oss.str());

	if( m_ElapsedTime < m_GoldTimeRef )
		m_txtTime.SetColor(Color::Yellow);
	else if( m_ElapsedTime < m_SilverTimeRef )
		m_txtTime.SetColor(Color(220, 220, 220));
	else if( m_ElapsedTime < m_BronzeTimeRef )
		m_txtTime.SetColor(Color(0x8C, 0x78, 0x53));
	else
		m_txtTime.SetColor(Color::Red);
}

void Level::RefreshSpeedFactor()
{
	ostringstream oss;
	oss << "Speed x "  << fixed << setprecision(1) << m_SpeedFactor;
	m_txtFactor.SetString(oss.str());
}

bool Level::NextPath()
{
	m_CurrentIndex++;

	if( m_PrevPath )
		delete m_PrevPath;
	m_PrevPath = m_CurrentPath;
	m_CurrentPath = m_NextPath;

	if( m_CurrentIndex + 1 < static_cast<int>(m_TabPathId.size()) )
	{
		m_NextPath = new CTexture(*Game::Get()->GetPathTexture(m_TabPathId[m_CurrentIndex + 1]));
		m_NextPath->SetPosition(800.f * (m_CurrentIndex + 1), 0.f);
	}
	else
		m_NextPath = NULL;

	return m_NextPath == NULL;
}

void Level::ButtonGoEffect(float elapsedTime)
{
	if( m_FadingGo > 0.f )
	{
		m_FadingGo -= elapsedTime;
		if( m_FadingGo < 0.f )
			m_FadingGo = 0.f;

		Color col = m_btnGo.GetColor();
		col.a = static_cast<sf::Uint8>(m_FadingGo * 255 / .5f);
		m_btnGo.SetColor(col);
		m_btnGo.SetScale((.5f - m_FadingGo) * 2.f + 1.f,
			(.5f - m_FadingGo) * 2.f + 1.f);
	}
}

void Level::RFEffect(float elapsedTime)
{
	if( m_TimerEffect > 0.f)
	{
		m_TimerEffect -= elapsedTime;
		if( m_TimerEffect < 0.f )
			m_TimerEffect = 0.f;

		Color col = m_CurrentRFEffect->GetColor();
		col.a = static_cast<sf::Uint8>(m_TimerEffect * 255 / .5f);
		m_CurrentRFEffect->SetColor(col);
		m_CurrentRFEffect->SetScale((.5f - m_TimerEffect) * 2.f + 1.f,
			(.5f - m_TimerEffect) * 2.f + 1.f);
	}
}
