#ifndef _LEVEL_H_
#define _LEVEL_H_

#include <SFML/Graphics.hpp>
#include <graphics/Texture.h>
#include <vector>
#include <list>

using namespace std;
using namespace sf;

class Obstacle;
class FrameImage;
class ProgressBar;
class Frame;

class Level
{
	View m_View;
	float m_ScrollSpeed;
	float m_SpeedFactor;
	Frame *m_SpeedFrame;
	Text m_txtFactor;

	static const float SPEED_INTERVAL;
	// Chrono
	float m_ElapsedTime;
	Text m_txtTime;
	Frame *m_ChronoFrame;
	bool m_RunChrono;

	// Dcrit le chemin du niveau (int -> index de texture chemin)
	vector<int> m_TabPathId;
	// index dans le tableau de path id
	int m_CurrentIndex;
	// Path textures currently displayed
	CTexture *m_PrevPath;
	CTexture *m_CurrentPath;
	CTexture *m_NextPath;

	float m_ElapsedDistance;

	// Reward / Forward effect
	bool m_RFEffect;
	float m_TimerEffect;
	Image m_ForwardImg;
	Sprite m_Forward;
	Image m_RewardImg;
	Sprite m_Reward;
	Sprite *m_CurrentRFEffect;

	// Obstacles
	list<Obstacle*> m_lObstacles;

	// Progress bar (progression in the level)
	FrameImage *m_imgProgressBar;
	ProgressBar *m_ProgressBar;
	Sprite m_Bar;

	// Go button for start
	bool m_Started;
	float m_FadingGo;
	CTexture m_btnGo;

	// Introduction screen and time references (in seconds)
	Frame *m_TimeFrame;
	float m_GoldTimeRef;
	Text m_txtGold;

	float m_SilverTimeRef;
	Text m_txtSilver;

	float m_BronzeTimeRef;
	Text m_txtBronze;

	// Defeat transition
	bool m_Defeat;
	float m_DefeatTimer;
	Image m_imgBlack;
	Sprite m_Fading;

public:
	Level(void);
	~Level(void);

	bool Init();
	void DeInit();

	bool FrameMove(float elapsedTime);
	void Render(RenderWindow &screen);

	int MouseMove(Vector2f pos);
	int MousePressed(Vector2f pos);

	// Up the speed
	void Forward();
	// Down the speed
	void Reward();

	const View& GetView();

// Private methods ====================================================
private:
	// Refresh text for the chronometer
	void RefreshChronometer(float elapsedTime);
	// Refresh text for the speed factor
	void RefreshSpeedFactor();

	// Load next path if needed
	bool NextPath();
	
	// Button Go effect animation
	void ButtonGoEffect(float elapsedTime);

	// Reward / Forward effect animation
	void RFEffect(float elapsedTime);
};

#endif
