#include "Obstacle.h"
#include <iostream>

Obstacle::Obstacle(CTexture *texture /*= NULL*/, Vector2f pos /*= Vector2f(0.f, 0.f)*/)
{
	if( texture != NULL)
	{
		SetTexture(texture);
		SetPosition(pos);
	}
}

Obstacle::~Obstacle(void)
{
	if(m_Texture)
		delete m_Texture;
}

void Obstacle::FrameMove(float /*elapsedTime*/)
{
	// Static obstacle
}

void Obstacle::Render(RenderWindow &screen)
{
	if(m_Texture)
		screen.Draw(*m_Texture);
}

// Detect collision with the obstacle (texture)
bool Obstacle::Collide(float x, float y)
{
	bool result = false;

	if( m_Texture )
	{
		Vector2f local = m_Texture->TransformToLocal(Vector2f(x, y));
		result = m_Texture->PointCollision(local);
	}

	return result;
}

void Obstacle::SetTexture(CTexture *texture)
{
	m_Texture = texture;
	m_Texture->SetOrigin(m_Texture->GetSize().x / 2.f,
		m_Texture->GetSize().y / 2.f);
}

void Obstacle::SetPosition(Vector2f pos)
{
	if(m_Texture)
	{
		m_Texture->SetPosition(pos);
	}
}
