#include <graphics/Frame.h>
#include <SFML/Graphics/Sprite.hpp>

using namespace sf;

class ProgressBar : public Frame
{
protected:
	float _Elapsed;	// Valeur coule
	float _Max;		// Valeur totale de la barre
	Vector2f _BarPos;	// Position de la barre de chargement dans le cadre

private:
	Sprite _spBar;	// Sprite reprsentant la barre de progression

	bool _ReductionEffectEnable;
	Image _imgReduction;	// Image blanche 1*1
	Sprite _spReduction;
	bool _ReductionEffect;	// Effet de rductionde la barre de temps en cours
	float _ReductionPos;	// Coordonne du positionnement de l'image dans la barre
	float _FadingSpeed;

public:
	ProgressBar( float x, float y, unsigned int width, unsigned int height, FrameImage *imgFrame,
		Sprite spBar, float max);
	~ProgressBar();

	// Set the progression of the bar to 0
	void Restart();

	// Animates the reduction effect if enabled
	bool FrameMove(float elapsed);

	// Displays the progress bar with the frame
	void Render(sf::RenderWindow &screen);
	void Render(sf::RenderWindow &screen, sf::Vector2f offset);

	// Returns true if the bar is full
	bool Add(float value);
	void Substract(float value);

	// Fading when the progress bar is substracted
	// Enable the effect and set the color of the fading
	void AddReductionEffect(Color col);

	// Enable or disable the reduction effect
	void EnableReductionEffect(bool enable);
	
	void SetMax( const float &value);
	float GetMax(void) const;

	float GetRemaining() const;

	void SetElapsed( float value);
	float GetElapsed() const;

	void SetFadingSpeed(const float fadingSpeed);
	float GetFadingSpeed() const;

private:
	// Scale the sprite representing the progression
	void ScaleProgressBar();
};

