#ifndef TEXTURE_H
#define TEXTURE_H

#include <SFML/Graphics.hpp>
#include <list>

typedef sf::Rect<unsigned int> uintRect;

// Remplace la couleur de certains pixels par une autre
void ReplaceColor( sf::Image &image, const sf::Color &filtre, const sf::Color &color, const uintRect &rect);

class CTexture : public sf::Sprite
{
	float m_Width,
		m_Height;
public:
	CTexture();
	CTexture( int x, int y, int longueur, int hauteur, const sf::Image &img );
	CTexture( const CTexture& texture);
	~CTexture();

	// Afficher la texture  l'endroit voulu
	void Render( sf::RenderWindow &screen );
	void Render( sf::RenderWindow &screen, float x, float y );
	void Render( sf::RenderWindow &screen, sf::Vector2f position );
	// Repoduction de la texture sur la largeur
	void Render( sf::RenderWindow &screen, float x, float y, int largeur/*, int hauteur*/ );
	void RenderWithOffset( sf::RenderWindow &screen, sf::Vector2f offset );

	// Redimensionne la texture avec les valeurs par dfauts
	void DefaultSize();

	inline void SetWidth( float width) { SetScaleX(width / m_Width ); }
	inline void SetHeight( float height) { SetScaleY(height / m_Height); }
	inline float GetDefaultWidth(void) { return m_Width; }
	inline float GetDefaultHeight(void) { return m_Height; }

	bool PointCollision( sf::Vector2i pt);
	bool PointCollision( sf::Vector2f pt);

};

typedef std::vector<CTexture*> TextureVector;

class CTextureVector : public TextureVector
{
	int m_Index;

public:
	CTextureVector();
	CTextureVector( int baseX, int baseY,	// Coordonnes de la 1re image
				  int nbFrames,			// Nombre d'image qui compose l'animation
				  const sf::Image &img,		// Image o sont regroups les frames de l'animation
				  int hauteur, int largeur ); // Dimension d'une frame

	~CTextureVector();
	// Positionnement sur l'axe des abscisses
	void SetX( float x);
	// Positionnement sur l'axe des ordonnes
	void SetY( float y);
	// Inverse la texture par rapport  l'axe des abscisses
	void FlipX( bool b);
	// Inverse la texture par rapport  l'axe des ordonnes
	void FlipY( bool b);
	// Dtermine le centre des textures (0,0) par dfaut
	void SetCenter( float x, float y);
};

#endif
