#ifndef BUTTON_H
#define BUTTON_H

#include <gui/StaticLabel.h>
#include <gui/WidgetImage.h>
#include <SFML/System.hpp>

/// <summary>
/// Un bouton permet de raliser des commandes
/// Il hrite du LabelStatic donc dispoe d'une image de fond et d'un texte (les deux facultatifs)
/// Une image supplmentaire peut tre ajout pour le sruvol de la souris
///
/// Il bnficie de toutes les notifications ( transmettre  l'objet)
/// </summary>
class Button : public StaticLabel
{
protected:
	WidgetImage *m_imgNormal;	// Rfrence vers l'image de fond habituel (shared data)
	WidgetImage *m_imgHover;	// Image au survol (shared data)
	CFont *m_pFontNormal;		// Rfrence vers la police habituel (shared data)
	CFont *m_pFontHover;		// Police du texte au survol (shared data)

public:
	inline void SetImageHover(WidgetImage *imgHover) { m_imgHover = imgHover; }
	inline void SetFontHover(CFont *fontHover) { m_pFontHover = fontHover; }

	/// <summary>
	/// Construit un bouton avec une position, une dimension, un identifiant (unique) et une image de fond
	/// </summary>
	/// <param name="x">position x du bouton</param>
	/// <param name="y">position y du bouton</param>
	/// <param name="width">largeur</param>
	/// <param name="height">hauteur</param>
	/// <param name="id">identifiant du widget</param>
	/// <param name="img">image du fond, peut valoir null (peu recommander)</param>
	Button(int x, int y, int width, int height, int id, WidgetImage *widgetImg);

	/// <summary>
	/// Construit un bouton avec une position, une dimension, un identifiant (unique) et une image de fond
	/// et en plus un texte
	/// </summary>
	Button(int x, int y, int width, int height, int id, WidgetImage *widgetImg,
		CFont *font, int tx, int ty, string texte);

	virtual ~Button();

	/// <summary>
	/// Affiche le bouton, voir Widget.cs pour plus d'infos
	/// </summary>
	virtual void Render(RenderWindow &screen, Vector2i offset);

	/// <summary>
	/// Collision du bouton, voir Widget.cs pour plus d'infos
	/// </summary>
	virtual bool Collide(int x, int y);

	virtual void SetWidgetImage(WidgetImage *img);
};

#endif
