#ifndef STATIC_LABEL_H
#define STATIC_LABEL_H

#include "Widget.h"
#include "WidgetImage.h"
#include <graphics/Font.h>
#include <SFML/System.hpp>

using namespace sf;
using namespace std;

/// <summary>
/// Label statique, permet uniquement d'afficher un texte et/ou une image, il ne dtecte aucune notification
/// Son identifiant est obligatoirement (-1 (Constant.CMD_NULL))
///
/// Le texte est facultatif
///		-il peut tre affich avec un dcalage par rapport au widget pour le positionner
/// L'image est facultative
/// L'image est de type WidgetImage [ O ](voir WidgetImage.h pour plus d'info)
/// </summary>
class StaticLabel : public Widget
{

protected:
	WidgetImage *m_Background;	// Image de fond utilise par le label (donne partage)
	Vector2i m_TextPosition;	// Position du texte par rapport au bouton
	CFont *m_pFont;				// Police du texte (donne partage)
	Text *m_DrawText;			// Texte affich et contenu
	Color m_TextColor;			// Couleur du texte

public:
	virtual void Enable( bool enable);

	// Texte du label
	inline void SetText( string text)	{ m_DrawText->SetString(text); }
	inline string GetText(void)			{ return m_DrawText->GetString(); }

	// Modifier/rcuprer la couleur du texte
	inline void SetColor( Color color)	{ m_TextColor = color; }
	inline Color GetColor(void)			{ return m_TextColor; }

	/// <summary>
	/// Construit un label statique avec une position, une dimension et une image de fond
	/// </summary>
	/// <param name="x">position x du label</param>
	/// <param name="y">position y du label</param>
	/// <param name="width">largeur</param>
	/// <param name="height">hauteur</param>
	/// <param name="img">image du fond, peut valoir null</param>
	StaticLabel(int x, int y, int width, int height, WidgetImage *img);

	/// <summary>
	/// Construit un label statique avec une position, une dimension et une image de fond
	/// </summary>
	/// <param name="x">position x du label</param>
	/// <param name="y">position y du label</param>
	/// <param name="width">largeur</param>
	/// <param name="height">hauteur</param>
	/// <param name="img">image du fond, peut valoir null</param>
	/// <param name="font">police utilise par le texte</param>
	/// <param name="tx">dcalage du texte sur l'axe x par rapport au label</param>
	/// <param name="ty">dcalage du texte sur l'axe y par rapport au label</param>
	/// <param name="texte">texte que contient le label</param>
	StaticLabel(int x, int y, int width, int height, WidgetImage *img,
		CFont *font, int tx, int ty, string texte);

	virtual ~StaticLabel();

	/// <summary>
	/// Affiche le label, voir Widget.cs pour plus d'infos
	/// </summary>
	virtual void Render( RenderWindow &screen, Vector2i offset);

	/// <summary>
	/// Aucun vnnement ne pet tre assign  un label statique
	/// </summary>
	virtual bool Collide(int x, int y);

	virtual void SetWidgetImage(WidgetImage *img);
};

#endif
