#ifndef WIDGET_H
#define WIDGET_H

#include <SFML/System.hpp>
#include <SFML/Graphics.hpp>
#include <graphics/GConstant.h>

using namespace sf;

class Widget
{
protected:
	Vector2i m_Position;	// Position relatif du widget
	Vector2i m_Size;		// Taille du widget
	bool m_Enable;				// Rceptionne les notifications (clic de souris, survol etc..)
								// un ID CMD_NULL ne peut jamais recevoir de notification
	bool m_Display;
	int m_Id;	// Identifiant de commande du bouton

	bool m_Hover;		// lment survol par la souris
	bool m_Pressed;		// lment cliqu par la souris

public:
	virtual bool IsEnabled();
	virtual void Enable(bool enable);

	virtual bool IsDisplayed();
	virtual void Display(bool display);

	int GetId(void) const;

	// Position ==
	inline int GetX() const { return m_Position.x; }
	inline int GetY() const { return m_Position.y; }

	virtual void SetX(int x) { m_Position.x = x; }
	virtual void SetY(int y) { m_Position.y = y; }

	// Dimension
	inline int GetWidth(void) { return m_Size.x; }
	inline int GetHeight(void) { return m_Size.y; }

	virtual void SetWidth(int width) { m_Size.x = width; }
	virtual void SetHeight(int height) { m_Size.y = height; }

	Widget(int id=CMD_NULL);
	/// <summary>
	/// Construit un widget avec une position et une dimension en pixel
	/// </summary>
	/// <param name="x">position relative sur l'axe x</param>
	/// <param name="y">position relative sur l'axe y</param>
	/// <param name="width">largeur de l'lment</param>
	/// <param name="height">hauteur de l'lment</param>
	/// <param name="id">identifiant du widget</param>
	Widget(int x, int y, int width, int height, int id=CMD_NULL);

	virtual ~Widget();

	/// <summary>
	/// Dtecte la collision avec le widget
	/// </summary>
	/// <param name="x">position relative par rapport  l'lment sur l'axe x</param>
	/// <param name="y">position relative par rapport  l'lment sur l'axe y</param>
	/// <returns>renvoie vrai s'il y a collision entre le sf::Vector2i et l'lment</returns>
	virtual bool Collide(int x, int y);

	/// <summary>
	/// Evnment d'un clic de souris destin  l'lment
	/// </summary>
	/// <param name="x">position x du clic relative  l'lment</param>
	/// <param name="y">position y du clic relative  l'lment</param>
	/// <returns>Renvoie l'identifiant de l'lement si le clic se situe sur celui-ci</returns>
	virtual int MousePressed(int x, int y);

	/// <summary>
	/// Evnement d'un relachement de clic de souris destin  l'lment
	/// </summary>
	/// <param name="x">position x du clic relative  l'lment</param>
	/// <param name="y">position y du clic relative  l'lment</param>
	/// <returns>Renvoie l'identifiant de l'lement si l'lment t enfonc et si la souris se situe sur celui-ci
	/// lors du relachement du bouton</returns>
	virtual int MouseReleased(int x, int y);

	/// <summary>
	/// Active le survol pour l'lment si le sf::Vector2i se situe  l'intrieur
	/// </summary>
	/// <param name="x">position x relative  l'lment</param>
	/// <param name="y">position y relative  l'lment</param>
	virtual void MouseHover(int x, int y);

	/// <summary>
	/// Affichage du widget, cette fonction doit obligatoirement tre implment pour dessiner le widget
	/// </summary>
	/// <param name="screen">Zone d'affichage</param>
	/// <param name="offset">Dcalage de l'lment (par rapport au pre)</param>
	virtual void Render(sf::RenderWindow &screen, sf::Vector2i offset) = 0;
};

#endif
