#ifndef MYLIB_WINDOW_H
#define MYLIB_WINDOW_H

#include "Widget.h"
#include <graphics/FrameImage.h>
#include <graphics/GConstant.h>
#include <SFML/Window.hpp>
#include <list>

using namespace std;
using namespace sf;

namespace myLib {
	class Window : public Widget
	{
	protected:
		FrameImage *m_Background;	// Fond de la fentre
		list<Widget*> m_Items;		// Contrles compris dans la fentre

	public:
		Window(int id=CMD_NULL);

		Window( int x, int y, int id=CMD_NULL);

		/// <summary>
		/// Construit une nouvelle fentre avec une position, une dimension et un identifiant donn
		/// </summary>
		/// <param name="x">position relative sur l'axe x</param>
		/// <param name="y">position relative sur l'axe y</param>
		/// <param name="width">largeur de la fentre</param>
		/// <param name="height">hauteur de la fentre</param>
		/// <param name="id">identifiant de la fentre</param>
		Window( int x, int y, int width, int height, int id=CMD_NULL);

		virtual ~Window();

		/// <summary>
		/// Affiche la fentre  l'cran, voir Widget.h pour plus d'infos
		/// </summary>
		virtual void Render(RenderWindow &screen, Vector2i offset);

		/// <summary>
		/// Clic sur l'cran, vrification si un widget a t cliqu.
		/// </summary>
		/// <param name="x">position du clic sur l'axe x</param>
		/// <param name="y">position du clic sur l'axe y</param>
		/// <returns>identifiant de l'lment cliqu, -1 sinon</returns>
		virtual int MousePressed(int x, int y);

		/// <summary>
		/// Relachement d'un clic de la souris, vrification si un widget est concern
		/// </summary>
		/// <param name="x">position x du clic relative  l'lment</param>
		/// <param name="y">position y du clic relative  l'lment</param>
		/// <returns>Renvoie l'identifiant d'un widget concern par l'vnement si l'lment t enfonc prcdemment
		/// et si la souris se situe sur celui-ci lors du relachement du bouton</returns>
		virtual int MouseReleased(int x, int y);

		/// <summary>
		/// Teste le survol de souris pour chaque widget faisant partie de la fentre
		/// </summary>
		/// <param name="x">position x relative  l'lment</param>
		/// <param name="y">position y relative  l'lment</param>
		virtual void MouseHover(int x, int y);

		/// <summary>
		/// Assigne un cadre de fond  la fentre, ce cadre est facultatif
		/// </summary>
		/// <param name="background">image du fond</param>
		void SetBackground(FrameImage *background);

		/// <summary>
		/// Renvoie le widget faisant partie de la fentre possdant tel identifiant
		/// </summary>
		/// <param name="id">identifiant unique du widget</param>
		/// <returns>le widget possdant l'identifiant id, null s'il n'existe pas</returns>
		Widget* GetWidget(int id);

		/// <summary>
		/// Ajoute le widget  la fentre
		/// </summary>
		/// <param name="widget">Contrle  ajouter</param>
		virtual void AddWidget(Widget *widget);

		/// <summary>
		/// Remove all widgets of the window && delete them
		/// </summary>
		void RemoveAllWidget();

		// Enable/Disable the window && all his components
		virtual void Enable(bool enable);
	};
};

#endif
