#include "graphics\BlinkEffect.h"

BlinkEffect::BlinkEffect(const Vector2f& pos, int width, int nbLines, const Color& col,
		float fadingSpeed, float scaleFactor, float spawnSpeed):
_NbLines		(nbLines * 2),
_FadingSpeed	(fadingSpeed),
_ScaleFactor	(scaleFactor),
_SpawnSpeed		(spawnSpeed),
_Timer			(0.f),
_SpFadingSpeed	(255 / (_NbLines * (1 / spawnSpeed)))
{
	_Lines = new Sprite[_NbLines * 2];
	_LineAlpha = new float[_NbLines * 2];
	_LineScale = new float[_NbLines * 2];
	_ImgLine.Create(width, 1, col);

	for( int i = 0; i < _NbLines / 2; i++)
	{
		_LineAlpha[i * 2] = 255.f;
		_LineScale[i * 2] = 0.4f;

		_Lines[i * 2].SetImage(_ImgLine);
		_Lines[i * 2].SetPosition(pos.x, pos.y + i * 4);
		_Lines[i * 2].SetOrigin(_Lines[i].GetSize().x / 2.f, _Lines[i].GetSize().y / 2.f);
		_Lines[i * 2].SetScaleY(_LineScale[i]);

		_LineAlpha[i * 2 + 1] = 255.f;
		_LineScale[i * 2 + 1] = 0.4f;

		_Lines[i * 2 + 1].SetImage(_ImgLine);
		_Lines[i * 2 + 1].SetPosition(pos.x, pos.y - i * 4);
		_Lines[i * 2 + 1].SetOrigin(_Lines[i].GetSize().x / 2.f, _Lines[i].GetSize().y / 2.f);
		_Lines[i * 2 + 1].SetScaleY(_LineScale[i]);
	}
}

BlinkEffect::~BlinkEffect(void)
{
	delete[] _Lines;
	delete[] _LineAlpha;
	delete[] _LineScale;
}

void BlinkEffect::Reset()
{
	_Timer = 0.f;

	Color col = _Sprite.GetColor();
	col.a = 0;
	_Sprite.SetColor(col);

	for( int i = 0; i < _NbLines; i++)
	{
		_LineAlpha[i] = 255.f;
		_LineScale[i] = 0.4f;
	}

}

void BlinkEffect::AddSprite(const Sprite& sp)
{
	_Sprite = sp;
	Color color = _Sprite.GetColor();
	color.a = 0;
	_Sprite.SetColor(color);
}

bool BlinkEffect::FrameMove(float elapsedTime)
{
	_Timer += elapsedTime;

	Color col;
	if( _Timer * _SpFadingSpeed < 255.f)
	{
		col = _Sprite.GetColor();
		col.a = static_cast<sf::Uint8>(_Timer * _SpFadingSpeed);
		_Sprite.SetColor(col);
	}

	for( int i = 0; i < _NbLines; i++)
	{
		for( int j = i * 2; j < (i + 1) * 2; j++)
		{
			if( _LineAlpha[j] != 0.f && _Timer >= (i * (1 / _SpawnSpeed)))
			{
				_LineAlpha[j] -= elapsedTime * _FadingSpeed;
				if( _LineAlpha[j] < 0.f )
					_LineAlpha[j] = 0.f;

				_LineScale[j] += elapsedTime * _ScaleFactor;

				col = _Lines[j].GetColor();
				col.a = static_cast<sf::Uint8>(_LineAlpha[j]);
				_Lines[j].SetColor(col);
				_Lines[j].SetScaleY(_LineScale[j]);
			}
		}
	}

	return _LineAlpha[_NbLines] == 0.f;
}

void BlinkEffect::Render(sf::RenderWindow &screen)
{
	screen.Draw(_Sprite);
	for( int i = 0; i < _NbLines; i++)
	{
		screen.Draw(_Lines[i]);
	}
}
