#include <graphics/Texture.h>
#include <iostream>

//=================================
// Extension des fonctions d'image
//=================================

void ReplaceColor( sf::Image &image, const sf::Color &filtre, const sf::Color &color, const uintRect &rect)
{
	for( unsigned int j=rect.Top; j<rect.Top + rect.Height; j++)
	{
		for( unsigned int i=rect.Left; i<rect.Left + rect.Width; i++)
		{
			if( image.GetPixel(i, j) == filtre )
			{
				image.SetPixel( i, j, color);
			}
		}
	}
}

//________________________________//
//||||||  Classe : Texture  ||||||//
////
CTexture::CTexture():
m_Width	(0),
m_Height(0)
{
}

CTexture::CTexture( int x, int y, int longueur, int hauteur, const sf::Image &img ):
sf::Sprite(img),
m_Width	((float)longueur),
m_Height((float)hauteur)
{
	SetSubRect(sf::IntRect( x, y, longueur, hauteur));
}

CTexture::CTexture( const CTexture& texture):
Sprite(texture)
{
	m_Width = texture.m_Width;
	m_Height = texture.m_Height;
}

CTexture::~CTexture()
{
}

void CTexture::Render( sf::RenderWindow &screen )
{
	screen.Draw(*this);
}

void CTexture::Render( sf::RenderWindow &screen, float x, float y )
{
	this->SetX(x);
	this->SetY(y);
	Render(screen);
}

void CTexture::Render( sf::RenderWindow &screen, sf::Vector2f position )
{
	this->SetX(position.x);
	this->SetY(position.y);
	Render(screen);
}

void CTexture::RenderWithOffset( sf::RenderWindow &screen, sf::Vector2f offset )
{
	sf::Vector2f pos = GetPosition();
	Move(offset);
	screen.Draw(*this);
	SetPosition(pos);
}

void CTexture::Render( sf::RenderWindow &screen, float x, float y, int largeur/*, int hauteur*/ )
{
	if( largeur <= 0 ) return;
	/*if( hauteur <= 0 ) return false;*/
	int larg = (int)GetSize().x;
	int haut = (int)GetSize().y;
	for( int i = 0; i<largeur-larg; i+=larg )
	{
		Render( screen, x+i, y);
	}
	if( largeur%larg != 0 )
	{
		sf::IntRect ir = GetSubRect();
		SetSubRect(sf::IntRect( ir.Left, ir.Top, ir.Left+largeur%larg, ir.Top + haut));
		Render( screen, x+largeur-largeur%larg, y);
		SetSubRect(ir);
	}
}

void CTexture::DefaultSize()
{
	Resize((float)m_Width, (float)m_Height);
}

bool CTexture::PointCollision( sf::Vector2i pt)
{
	sf::IntRect ir = GetSubRect();
	if( pt.x >= ir.Width ) return false;
	if( pt.x < 0 ) return false;
	if( pt.y >= ir.Height ) return false;
	if( pt.y < 0 ) return false;

	sf::Color col = GetImage()->GetPixel(pt.x + ir.Left, pt.y + ir.Top);
	return col.a > 100;
}

bool CTexture::PointCollision( sf::Vector2f pt)
{
	sf::IntRect ir = GetSubRect();
	if( pt.x >= (float)ir.Width ) return false;
	if( pt.x < 0.f ) return false;
	if( pt.y >= (float)ir.Height ) return false;
	if( pt.y < 0.f ) return false;

	sf::Color col = GetImage()->GetPixel((int)pt.x + ir.Left, (int)pt.y + ir.Top);
	return col.a > 100;
}

//		_______________________________
//		|||| o Liste de textures o ||||
//		
CTextureVector::CTextureVector()
{
	m_Index = 0;
}

CTextureVector::CTextureVector( int baseX, int baseY, int nbFrames, const sf::Image &img,
			int hauteur, int largeur )
{
	CTexture *texture;
	if( nbFrames <= 0 )
	{
		std::cerr <<  "[Erreur] - Animation vide" << std::endl;
		return;
	}

	if( !empty() )	clear();

	for( int i=0; i<nbFrames; i++)
	{
		texture = new CTexture( baseX+i*largeur, baseY, largeur, hauteur, img);
		push_back(texture);
	}

	m_Index = 0;	// Liste en position initiale
}

CTextureVector::~CTextureVector()
{
	CTexture *pText = NULL;
	for( unsigned int i = 0; i < size(); i++)
	{
		pText = at(i);
		delete pText;
	}
	clear();
}

void CTextureVector::SetX( float x)
{
	for( unsigned int i = 0; i < size(); i++)
		at(i)->SetX(x);
}
void CTextureVector::SetY( float y)
{
	for( unsigned int i = 0; i < size(); i++)
		at(i)->SetY(y);
}

void CTextureVector::FlipX( bool b)
{
	for( unsigned int i = 0; i < size(); i++)
		at(i)->FlipX(b);
}

void CTextureVector::FlipY( bool b)
{
	for( unsigned int i = 0; i < size(); i++)
		at(i)->FlipY(b);
}

void CTextureVector::SetCenter( float x, float y)
{
	for( unsigned int i = 0; i < size(); i++)
		at(i)->SetOrigin( x, y);
}
