#include <gui/StaticLabel.h>
#include <graphics/GConstant.h>

StaticLabel::StaticLabel(int x, int y, int width, int height, WidgetImage *img):
Widget(x, y, width, height, CMD_NULL),
m_Background(img),
m_pFont		(NULL),
m_DrawText	(NULL)
{
}

StaticLabel::StaticLabel(int x, int y, int width, int height, WidgetImage *img,
	CFont *font, int tx, int ty, string text):
Widget(x, y, width, height, CMD_NULL)
{
	m_Background = img;
	m_TextPosition = Vector2i(tx, ty);
	m_pFont = font;
	m_DrawText = m_pFont->MakeString(text);
}

StaticLabel::~StaticLabel()
{
	if(m_DrawText)
		delete m_DrawText;
}

void StaticLabel::Enable( bool enable)
{
	Color color = m_DrawText->GetColor();
	if(enable && !m_Enable)
		m_DrawText->SetColor( Color( color.r * 2, color.g * 2, color.b * 2));
	else if( !enable && m_Enable)
		m_DrawText->SetColor( Color( color.r / 2, color.g / 2, color.b / 2));
	Widget::Enable(enable);
}

void StaticLabel::Render( RenderWindow &screen, Vector2i offset)
{
	if (m_Display)
	{
		offset.x += m_Position.x;
		offset.y += m_Position.y;
		// Dessine le fond du label
		if (m_Background)
			m_Background->Render(screen, Vector2f((float)offset.x, (float)offset.y), (float)m_Size.x, (float)m_Size.y, m_Enable);

		if (m_pFont)
		{
			// TODO: intercepter SetHover
			m_DrawText->SetFont(*m_pFont);
			m_DrawText->SetColor(m_pFont->GetColor());
			// Afiche le texte du label
			m_DrawText->SetX((float)(offset.x + m_TextPosition.x));
			m_DrawText->SetY((float)(offset.y + m_TextPosition.y));
			screen.Draw(*m_DrawText);
		}
	}
}

bool StaticLabel::Collide(int /*x*/, int /*y*/)
{
	return false;
}

void StaticLabel::SetWidgetImage(WidgetImage *img)
{
	m_Background = img;
}
