#include <gui/Widget.h>
#include <graphics/GConstant.h>

Widget::Widget(int id):
m_Position	(Vector2i(0,0)),
m_Size		(Vector2i(0,0)),
m_Enable	(true),
m_Display	(true),
m_Id		(id),
m_Hover		(false),
m_Pressed	(false)
{

}

Widget::Widget(int x, int y, int width, int height, int id):
m_Position	(Vector2i(x, y)),
m_Size		(Vector2i(width, height)),
m_Enable	(true),
m_Display	(true),
m_Id		(id),
m_Hover		(false),
m_Pressed	(false)
{
}

Widget::~Widget()
{

}

bool Widget::IsEnabled()
{
	return m_Enable;
}

void Widget::Enable(bool enable)
{
	m_Enable = enable;
}

bool Widget::IsDisplayed()
{
	return m_Display;
}

void Widget::Display(bool display)
{
	m_Display = display;
}

int Widget::GetId(void) const
{
	return m_Id;
}

bool Widget::Collide(int x, int y)
{
	if (x < m_Position.x) return false;   //. []
	if (x >= m_Position.x + m_Size.x) return false;   //[] .
	if (y < m_Position.y) return false;   //. =
	if (y >= m_Position.y + m_Size.y) return false;	//= .

	return true;
}

int Widget::MousePressed(int x, int y)
{
	if (!m_Enable || m_Id == CMD_NULL)
		return CMD_NULL;
	//else
	int id = CMD_NULL;
	if (Collide(x, y))
	{
		m_Pressed = true;
		id = m_Id;
	}
	else
		m_Pressed = false;
	return id;
}

int Widget::MouseReleased(int x, int y)
{
	if (!m_Enable || m_Id == CMD_NULL)
		return CMD_NULL;
	// else
	int id = CMD_NULL;
	if (m_Pressed)
	{
		if (Collide(x, y))
		{
			id = m_Id;
		}
	}
	m_Pressed = false;
	return id;
}

void Widget::MouseHover(int x, int y)
{
	if (!m_Enable || m_Id == CMD_NULL)
		return;
	//else
	m_Hover = Collide(x, y);
}
