#include <gui/WidgetImage.h>

WidgetImage::WidgetImage()
{

}

WidgetImage::WidgetImage(IntRect rect, Vector2i sides, const Image& img)
{
	/* Info:
	rect.Right = width
	rect.Bottom = height*/

	int leftSide = sides.x;
	int rightSide = sides.y;

	m_imgLeft = CTexture(rect.Left, rect.Top, leftSide, rect.Height, img);
	m_imgCenter = CTexture(rect.Left + leftSide, rect.Top, rect.Width - leftSide - rightSide, rect.Height, img);
	m_imgRight = CTexture(rect.Left + rect.Width - rightSide, rect.Top, rightSide, rect.Height, img);
}

WidgetImage::~WidgetImage()
{
}

void WidgetImage::Render(RenderWindow &screen, Vector2f position, float width, float height, bool enable)
{
	Color col = m_imgLeft.GetColor();
	if (!enable && col != Color(100, 100, 100, col.a))
	{
		m_imgLeft.SetColor( Color(100, 100, 100, col.a));
		m_imgCenter.SetColor( Color(100, 100, 100, col.a));
		m_imgRight.SetColor( Color(100, 100, 100, col.a));
	}
	else if (enable && col != Color(255, 255, 255, col.a))
	{
		m_imgLeft.SetColor( Color(255, 255, 255, col.a));
		m_imgCenter.SetColor( Color(255, 255, 255, col.a));
		m_imgRight.SetColor( Color(255, 255, 255, col.a));
	}

	m_imgLeft.SetHeight(height);
	m_imgLeft.Render(screen, position);
	m_imgCenter.Resize(width - m_imgLeft.GetSize().x - m_imgRight.GetSize().x, height);
	m_imgCenter.Render(screen, Vector2f(position.x + m_imgLeft.GetSize().x, position.y));
	m_imgRight.SetHeight(height);
	m_imgRight.Render(screen, Vector2f(position.x + (width - m_imgRight.GetSize().x), position.y));
}

void WidgetImage::SetColorAlpha(sf::Uint8 alpha)
{
	Color col = m_imgLeft.GetColor();
	m_imgLeft.SetColor(	Color(col.r, col.g, col.b, alpha));
	m_imgCenter.SetColor(	Color(col.r, col.g, col.b, alpha));
	m_imgRight.SetColor(	Color(col.r, col.g, col.b, alpha));
}

bool WidgetImage::Collide(float x, float y, float width, float height)
{
	// Gauche
	Vector2f scale = Vector2f( 1.0f, height / m_imgLeft.GetDefaultHeight());
	bool b = m_imgLeft.PointCollision( Vector2f( x, y / scale.y) );
	// Droite
	b = b || m_imgRight.PointCollision( Vector2f(x - (width - m_imgRight.GetSize().x),
												y / scale.y) );
	// Centre, si la largeur de l'image central est suprieur  0
	if( m_imgCenter.GetDefaultWidth() != 0.0f)
	{
		scale.x = (width - m_imgLeft.GetSize().x - m_imgRight.GetSize().x) / m_imgCenter.GetDefaultWidth();
		b = b || m_imgCenter.PointCollision( Vector2f((x - m_imgLeft.GetSize().x) / scale.x,
													y / scale.y) );
	}
	return b;
}
