#include <gui/Window.h>
#include <graphics/GConstant.h>

namespace myLib {

Window::Window(int id):
Widget(id),
m_Background	(NULL)
{
}

Window::Window( int x, int y, int id):
Widget(x, y, 0, 0, id),
m_Background	(NULL)
{

}

Window::Window( int x, int y, int width, int height, int id):
Widget(x, y, width, height, id),
m_Background	(NULL)
{
}

Window::~Window()
{
	while( !m_Items.empty())
	{
		delete m_Items.front();
		m_Items.pop_front();
	}
}

void Window::Render(RenderWindow &screen, Vector2i offset)
{
	if( m_Display)
	{
		if (m_Background != NULL)
		{
			if( m_Enable != m_Background->IsEnabled())
				m_Background->Enable(m_Enable);
			m_Background->Render(screen, (float)(offset.x + m_Position.x),
										(float)(offset.y + m_Position.y),
										m_Size.x,
										m_Size.y);
		}

		list<Widget*>::iterator it;
		for( it = m_Items.begin(); it != m_Items.end(); it++)
			(*it)->Render(screen, offset + m_Position);
	}
}

int Window::MousePressed(int x, int y)
{
	int id = -1;
	if(m_Display && m_Enable)
	{
		list<Widget*>::iterator it = m_Items.begin();
		while( it != m_Items.end() && id == -1)
		{
			id = (*it)->MousePressed(x - m_Position.x, y - m_Position.y);
			it++;
		}
	}

	return id;
}

int Window::MouseReleased(int x, int y)
{
	int id = -1;
	if(m_Display && m_Enable)
	{
		list<Widget*>::iterator it = m_Items.begin();
		while( it != m_Items.end() && id == -1)
		{
			id = (*it)->MouseReleased(x - m_Position.x, y - m_Position.y);
			it++;
		}
	}

	return id;
}

void Window::MouseHover(int x, int y)
{
	if(m_Display && m_Enable)
	{
		list<Widget*>::iterator it;
		for( it = m_Items.begin(); it != m_Items.end(); it++)
			(*it)->MouseHover(x - m_Position.x, y - m_Position.y);
	}
}

void Window::SetBackground(FrameImage* background)
{
	m_Background = background;
}

Widget* Window::GetWidget(int id)
{
	// Les lments sans identifiant ne sont pas accessibles
	if (id == CMD_NULL)
		return NULL;
	//else
	Widget* pWidget = NULL;
	list<Widget*>::iterator it;
	for( it = m_Items.begin(); it != m_Items.end(); it++)
	{
		pWidget = (*it);
		if (pWidget->GetId() == id)
			return pWidget;
	}
	return NULL;
} // GetWidget


void Window::AddWidget(Widget* pWidget)
{
	m_Items.push_front(pWidget);
	if(!m_Enable)
		pWidget->Enable(m_Enable);
}

void Window::RemoveAllWidget()
{
	while( !m_Items.empty())
	{
		delete m_Items.front();
		m_Items.pop_front();
	}
}

void Window::Enable(bool enable)
{
	if( (m_Enable && !enable) || (!m_Enable && enable))
	{
		list<Widget*>::iterator it;
		for( it = m_Items.begin(); it != m_Items.end(); it++)
			(*it)->Enable(enable);

		m_Enable = enable;
	}
} // Enable

}; // namespace myLib
